* Program file for "Disentangling Diverse Measures: A Survey of Financial Stress Indexes"
* by Kevin L. Kliesen, Michael T. Owyang, and E. Katarina Vermann
* Published in the Federal Reserve Bank of St. Louis Review September/October 2012, Volume 94 Number 4.

* Sources for series:
* Carlson Index: Proprietary. From Carlson, Mark; Lewis, Kurt and Nelson, William. Using Policy Intervention to Identify Financial Stress. Finance and Economics Discussion Series No. 2012-02, Board of Governors of the Federal Reserve System, 2012; www.federalreserve.gov/pubs/feds/2012/201202/201202pap.pdf.
* Bloomberg FCI: Proprietary, Bloomberg Terminal
* Goldman Sachs FCI: Proprietary, Bloomberg Terminal
* Cleveland FSI: http://www.clevelandfed.org/research/data/financial_stress_index/index.cfm
* Chicago NFCI & ANFCI: http://www.chicagofed.org/webpages/research/data/nfci/background.cfm
* St Louis FSI: http://research.stlouisfed.org/fred2/series/STLFSI
* Kansas City FSI: http://www.kansascityfed.org/research/indicatorsdata/kcfsi/
* All IMF FSIs: Proprietary. From Danninger, Stephan; Balakrishnan, Ravi; Elekdag, Selim and Tytell, Irina. How Linkages Fuel the Fire: The Transmission of Financial Stress from Advanced to Emerging Economies, in World Economic Outlook: Crisis and Recovery. Chap. 4. Washington, DC: International Monetary Fund, 2009, pp. 139-75; www.imf.org/external/pubs/ft/weo/2009/01/pdf/c4.pdf and 
* Greece FSI: Proprietary.Louzis, Dimitrios P. and Vouldis, Angelos T. A Financial Systemic Stress Index for Greece. Presented at the First Conference of the Macro-prudential Research (MaRs) network of the European System of Central Banks in Frankfurt am Main, October 2011
* Sweden FCI: Proprietary. Sandahl, Johannes F.; Holmfeldt, Mia; Rydn, Anders and Strmqvist, Maria. An Index of Financial Stress for Sweden. Sveriges Riksbank Economic Review, July 10, 2011, 2011(2), pp. 49-67;
* Bank of Canada FCI: Proprietary, Bloomberg Terminal


* Insert current directory here:
** cd "C:\.....\"

 * Table 4 *
use daily_frequency, clear
collapse cfsi , by(year month)
save daily_aggregated, replace

* Figure 1*

use weekly_frequency, clear
tsset date
format date %td
tsline stlfsi nfci if stlfsi ~=.
graph save figure1, replace

*** Table 5 ***
merge date using daily_frequency, sort
replace year = year(date)
replace week = week(date)
replace month = month(date)
collapse nfci anfci stlfsi cfsi   , by(year week)
mkcorr nfci anfci stlfsi cfsi   , l log(Table5) replace

* Table 6 *
use weekly_frequency, clear
replace month=month(date)
replace year = year(date)
collapse nfci anfci stlfsi adsbci, by (year month)
save weekly_aggregated, replace

use monthly_frequency, clear
*Note: for untruncated IP and Wilshire data, use monthly_frequency2 instead
sort year month
merge year month using daily_aggregated, _merge(merge_daily)
sort year month
merge year month using weekly_aggregated, _merge(merge_weekly)
replace date = ym(year, month)
gen commonsample = (year>=2000 & year<=2011)
replace commonsample = 0 if (year==2011 & month==12)


format date %tm

label var cfsi "Cleveland FSI"
label var nfci "Chicago NFCI"
label var anfci "Chicago Adjusted NFCI"
label var stlfsi "St. Louis FSI"
label var adsbci "Arouba-Diebold-Scotti Business Conditions Index"

save monthly_analysis, replace

mkcorr   kcfsi nfci anfci stlfsi   cfsi , l log(Table6) replace

* Table 11, Column 1 - Full Sample *

mkcorr adsbci  kcfsi nfci anfci stlfsi   cfsi , l log(Table11col1_full) replace 

* Figure 2 *

foreach g in stlfsi kcfsi cfsi nfci anfci     {
nbercycles `g' if `g'~=., file(recessionbars_`g'.do) replace
graph save `g', replace
}


gr combine stlfsi.gph kcfsi.gph cfsi.gph nfci.gph anfci.gph, commonscheme
graph save Figure2, replace


* Figure 4 *

* Note: This figure was originally made in excel using the IP growth series * 100 and STLFSI series. The trendline, equation, and R-squared were added using the "Add Trendline" menu

gen ip100 = ip*100
qui reg ip100 stlfsi
predict yhat
label var yhat "Trendline in Figure 4"
twoway (scatter ip100 stlfsi) (line yhat stlfsi, sort)
graph save Figure4, replace

* Figure 5 *

mvcorr kcfsi stlfsi, gen(kc_stl_2yrcorr) w(24) force
tsline kc_stl_2yrcorr
graph save figure5, replace

* Table Appendix B, Column 1: IP Lag Lengths *

foreach var in kcfsi  cfsi nfci anfci stlfsi    {
	varsoc ip `var', maxlag(6)
	}


* IP Full Sample Estimation *

tsset date


***2 lag***

foreach var in    cfsi stlfsi  {
keep ip `var' date
format date %tm
drop if (ip==. | `var'==.)

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var ip `var' in 1/`i', lag(1/2)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	

	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_full_`var', replace
use monthly_analysis, clear
tsset date
} 



***3 lag***

foreach var in   anfci nfci    {
keep ip `var' date
format date %tm
drop if (ip==. | `var'==.)

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var ip `var' in 1/`i', lag(1/3)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	
	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_full_`var', replace
use monthly_analysis, clear
tsset date
} 


**** 4 lag *****
foreach var in  kcfsi {
keep ip `var' date
format date %tm
drop if (ip==. | `var'==.)
 
tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12' {
	qui var ip `var' in 1/`i', lag(1/4)
	
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}



	forvalues h=1/12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_full_`var', replace
use monthly_analysis, clear
tsset date
} 


* Table 8 - Full Sample *

use rmse_ip_full_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon using rmse_ip_full_`var', nogen
}



outsheet _all using rmse_ip_full.csv, comma replace
save rmse_ip_full, replace

* IP Common Sample  *

***1 lag
use monthly_analysis, clear
drop if year<2000
drop if year>2011
drop if (year==2011 & month==12)
tsset date
save monthly_analysis_common, replace


* Table 11, Column 1 - Common Sample *

mkcorr adsbci  kcfsi nfci anfci stlfsi   cfsi , l log(Table11col1_common) replace 

* IP Common Sample - Estimation *



***2 lag***

foreach var in    cfsi stlfsi  {
keep ip `var' date
format date %tm

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var ip `var' in 1/`i', lag(1/2)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	

	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_common_`var', replace
use monthly_analysis_common, clear
tsset date
} 



***3 lag***

** NOTE: The estimates of the GSFCI and ANFCI in the common sample panel of Table 8 do not use the common sample.
** To reproduce results in Table 8 with the public use data, common sample panel, uncomment out lines 371 and 372.

*use monthly_analysis, clear
*tsset date
foreach var in   anfci nfci    {
keep ip `var' date 
format date %tm

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var ip `var' in 1/`i', lag(1/3)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	
	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_common_`var', replace
use monthly_analysis_common, clear
tsset date
} 


**** 4 lag *****
foreach var in  kcfsi {
keep ip `var' date 
format date %tm
 
tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12' {
	qui var ip `var' in 1/`i', lag(1/4)
	
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}



	forvalues h=1/12 {
		gen resid_`h' = (h`h'-ip)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
keep rmse*
collapse rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_ip_common_`var', replace
use monthly_analysis_common, clear
tsset date
} 

* Table 8 - IP Common Sample *

use rmse_ip_common_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon using rmse_ip_common_`var', nogen
}



outsheet _all using rmse_ip_common.csv, comma replace
save rmse_ip_common, replace

*************************************
* Forecasts with Wilshire ***********
*************************************

use monthly_analysis, clear
tsset date

* Appendix B, Column 3 (Wilshire Estimation Lag Lengths)

foreach var in kcfsi  cfsi nfci anfci stlfsi    {
	varsoc wilshire `var', maxlag(6)
	}

* Table 10 *

* Wilshire Full Sample - Estimation *

***1 lag
foreach var in kcfsi  cfsi anfci stlfsi   {
keep date wilshire `var'
drop if (wilshire==. | `var'==.)
format date %tm
 
tsappend, add(12)
	
gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .
 
local win 60
forv i = 60/`=_N-12' {
	qui var wilshire `var' in 1/`i', lag(1)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}

	
	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-wilshire)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_will_full_`var', replace
use monthly_analysis, clear
tsset date
}

***2 lag***

foreach var in nfci    {
keep wilshire `var' date
format date %tm
drop if (wilshire ==. | `var'==.)

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var wilshire `var' in 1/`i', lag(1/2)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	

	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-wilshire)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_will_full_`var', replace
use monthly_analysis, clear
tsset date
} 

* Table 10 - Full Sample *

use rmse_will_full_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon using rmse_will_full_`var', nogen
}



outsheet _all using rmse_will_full.csv, comma replace
save rmse_will_full, replace

* Wilshire Common Sample - Estimation *

use monthly_analysis_common, clear

***1 lag
foreach var in kcfsi  cfsi anfci stlfsi   {
keep date wilshire `var'
drop if (wilshire==. | `var'==.)
format date %tm
 
tsappend, add(12)
	
gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .
 
local win 60
forv i = 60/`=_N-12' {
	qui var wilshire `var' in 1/`i', lag(1)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}

	
	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-wilshire)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_will_common_`var', replace
use monthly_analysis_common, clear
tsset date
}

***2 lag***

foreach var in nfci    {
keep wilshire `var' date
format date %tm
drop if (wilshire ==. | `var'==.)

	tsappend, add(12)

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .
gen h5 = .
gen h6 = .
gen h7 = .
gen h8 = .
gen h9 = .
gen h10=.
gen h11 = .
gen h12 = .

local win 60
forv i = 60/`=_N-12'{
	qui var wilshire `var' in 1/`i', lag(1/2)
	predict yhat
		forvalues h=1/12 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat
	}
	

	forvalues h=1(1)12 {
		gen resid_`h' = (h`h'-wilshire)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
collapse (mean) rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
gen horizon = regexs(0) if (regexm(_varname, "[0-9][0-9]"))
destring horizon1, replace
destring horizon, replace
replace horizon = horizon1 if horizon==.
drop horizon1
rename v1 `var'
save rmse_will_common_`var', replace
use monthly_analysis_common, clear
tsset date
} 

* Table 10 - Common Sample *

use rmse_will_common_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon using rmse_will_common_`var', nogen
}


outsheet _all using rmse_will_common.csv, comma replace
save rmse_will_common, replace

*******************************
* Quarterly Data Analysis *****
*******************************

use monthly_analysis, clear
collapse *fsi *fci, by (year quarter)
merge year quarter using quarterly_indicators

*** Blanks for incomplete quarters ***
replace kcfsi = . if year == 1990 & quarter ==1
replace stlfsi = . if year == 1993 & quarter ==4

drop if year>2011

gen date = yq(year, quarter)
tsset date
save quarterly_analysis, replace

* Table 10, column 2 *

mkcorr sloo  kcfsi nfci anfci stlfsi   cfsi , l log(Table11col2_full) replace 

*  Appendix B: Column 2 *


foreach var in kcfsi  cfsi nfci anfci stlfsi    {
	varsoc gdpgrowth `var', maxlag(6)
	}

* GDP Growth Estimations - Full *

* 2 lag *

foreach var in cfsi nfci anfci stlfsi kcfsi       {
keep gdpgrowth `var' date
drop if gdpgrowth==.
drop if `var'==.

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .

local win 20
tsappend, add(4)

forv i = 20/`=_N-4' {
	qui var gdpgrowth `var' in 1/`i', lag(1/2)
	
	predict yhat
		forvalues h=1/4 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat

	}


	forvalues h=1/4 {
		gen resid_`h' = (h`h'-gdpgrowth)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
keep rmse*
collapse rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
destring horizon, replace
rename v1 `var'
save rmse_gdp_full_`var', replace
use quarterly_analysis, clear
tsset date
format date %tm
}


* Table 9 - Full Sample *

use rmse_gdp_full_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon rmse_gdp_full_`var', nogen
}



outsheet _all using rmse_gdp_full.csv, comma replace
save rmse_gdp_full, replace

* GDP Growth  - Common *

use quarterly_analysis, clear
drop if year<2000
drop if year>2011
drop if (year==2011 & quarter==4)
save quarterly_analysis_common, replace

* Table 10, column 2 *

mkcorr sloo  kcfsi nfci anfci stlfsi   cfsi , l log(Table11col2_common) replace 

* Estmation *

* 2 lag *

foreach var in cfsi nfci anfci stlfsi kcfsi       {
keep gdpgrowth `var' date
drop if gdpgrowth==.
drop if `var'==.

gen h1 = .
gen h2 = .
gen h3 = .
gen h4 = .

local win 20
tsappend, add(4)

forv i = 20/`=_N-4' {
	qui var gdpgrowth `var' in 1/`i', lag(1/2)
	
	predict yhat
		forvalues h=1/4 {
			local obs = `i'+`h'
			replace h`h' = yhat in `obs'
			}
	drop yhat

	}


	forvalues h=1/4 {
		gen resid_`h' = (h`h'-gdpgrowth)^2
		egen rmse_`h' = mean(resid_`h')
		}
		
keep rmse*
collapse rmse*
xpose, v clear
gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
destring horizon, replace
rename v1 `var'
save rmse_gdp_common_`var', replace
use quarterly_analysis_common, clear
tsset date
format date %tm
}

** NOTE: The Estimations for the KCFSI, NFCI, ANFCI, and GSFCI were (incorrectly) estimated with 3 lags
** To reproduce the results in the common sample relative RMSEs in Table 9 with public use data, uncomment out the  lines 894-935.

*foreach var in  nfci anfci  kcfsi       {
*keep gdpgrowth `var' date
*drop if gdpgrowth==.
*drop if `var'==.

*gen h1 = .
*gen h2 = .
*gen h3 = .
*gen h4 = .

*local win 20
*tsappend, add(4)

*forv i = 20/`=_N-4' {
	*qui var gdpgrowth `var' in 1/`i', lag(1/3)
	
	*predict yhat
		*forvalues h=1/4 {
			*local obs = `i'+`h'
			*replace h`h' = yhat in `obs'
			*}
	*drop yhat

	*}


	*forvalues h=1/4 {
		*gen resid_`h' = (h`h'-gdpgrowth)^2
		*egen rmse_`h' = mean(resid_`h')
		*}
		
*keep rmse*
*collapse rmse*
*xpose, v clear
*gen horizon1 = regexs(0) if (regexm(_varname, "[0-9]"))
*destring horizon, replace
*rename v1 `var'
*save rmse_gdp_common_`var', replace
*use quarterly_analysis_common, clear
*tsset date
*format date %tm
*}

* Table 9 - Common Sample *

use rmse_gdp_common_cfsi, clear
foreach var in     anfci nfci  stlfsi  kcfsi {
merge 1:1 horizon using rmse_gdp_common_`var', nogen
}



outsheet _all using rmse_gdp_common.csv, comma replace
save rmse_gdp_common, replace


**NOTE: The data to reproduce figure 3 and table 7 are proprietary. To see the code for figure 3 and table 7, see 1210klkp_proprietary.do
